# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-10-17 13:29
from __future__ import unicode_literals

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations
from django.db import models

import orchestra.models.core.mixins
import orchestra.utils.models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0069_update_currencies'),
    ]

    operations = [
        migrations.CreateModel(
            name='Todo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('is_deleted', models.BooleanField(default=False)),
                ('description', models.CharField(max_length=200)),
                ('completed', models.BooleanField(default=False)),
                ('task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='todos', to='orchestra.Task')),
            ],
            bases=(orchestra.models.core.mixins.TodoMixin, orchestra.utils.models.DeleteMixin, models.Model),
        ),
        migrations.AlterField(
            model_name='taskassignment',
            name='worker',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='assignments', to='orchestra.Worker'),
        ),
    ]
