# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-10-25 09:08
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0072_migrate_staffing_priority'),
    ]

    operations = [
        migrations.RemoveField(  # manually-reviewed
            model_name='worker',
            name='staffing_priority',
        ),
    ]
