import React from 'react'
import {
    RotateForward
} from '@b12/metronome/components/Icons.es6.js'

import './AnimatedCircle.scss'

const AnimatedCircle = () => {
    return (
      <div className="animated-circle">
        <RotateForward />
      </div>
    )
}

export default AnimatedCircle
