.list-by-status {
  padding-top: 20px;
  h4 {
    padding-bottom: 10px;
  }
  .todo {
    margin-left: 18px;
    input[type=checkbox] {
      transform: scale(1.1);
      margin-left: -22px;
    }
    label {
      align-items: end;
      display: inline;
    }
    &__role {
      font-weight: 500;
      padding-left: 5px;
    }
    &__dates {
      padding-left: 5px;
      font-style: italic;
      color: #999;
      &-danger {
        color: #a94442;
      }
      &-separator {
        font-style: normal;
      }
    }
    &-qa {
      margin-bottom: 0;
      .panel-body {
        padding: 5px;
        font-size: x-small;
        font-weight: normal;
      }
    }
  }
  .angular-ui-tree-node {
      padding-top: 12px;
      padding-bottom: 0;
  }

  .skipped-todos {
    margin-left: -18px;
  }

  .fa-circle-o {
    font-size: xx-small;
    vertical-align: middle;
    margin-left: -18px;
  }

  .angular-ui-tree-handle > label{
    cursor: pointer;
  }
}
