self.__precacheManifest = (self.__precacheManifest || []).concat([
  {
    "revision": "4120908d6b19c09319262120a0063c4e",
    "url": "/static/dist2/index.html"
  },
  {
    "revision": "c56dd44222c2cdf6a91c",
    "url": "/static/dist2/static/css/2.7cafca44.chunk.css"
  },
  {
    "revision": "a232180e72e0ac68059c",
    "url": "/static/dist2/static/css/main.ff3ce63a.chunk.css"
  },
  {
    "revision": "c56dd44222c2cdf6a91c",
    "url": "/static/dist2/static/js/2.564c31ea.chunk.js"
  },
  {
    "revision": "1aea4192c69f3efa45ca8d60d4c9b77a",
    "url": "/static/dist2/static/js/2.564c31ea.chunk.js.LICENSE.txt"
  },
  {
    "revision": "a232180e72e0ac68059c",
    "url": "/static/dist2/static/js/main.ea77b330.chunk.js"
  },
  {
    "revision": "0317add2a7aae2dc6c79",
    "url": "/static/dist2/static/js/runtime-main.30a1ae39.js"
  }
]);