# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-08-21 18:04
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0080_completed_project_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='step',
            name='todolist_templates_to_apply',
            field=models.ManyToManyField(blank=True, to='orchestra.TodoListTemplate'),
        ),
    ]
