.todo-list {
  .section-body {
    padding-top: 1.5rem;
    padding-bottom: 1.5rem;
  }

  .todolist-pane {
    border: 1px solid black;
    padding: 1.5rem;
    &__heading {
      margin-top: -2.8rem;
      background-color: white;
      text-align: center;
    }
  }

  .new-todo {
    &__datetime {
      display: inline-block;
      padding-left:10px;
    }
    select.form-control {
      border: 0;
      outline: 1px solid #cecece;
      outline-offset: -1px;
      box-shadow: none;
    }
    .form-control, .button {
      border-radius: 0;
      -webkit-border-radius: 0;
      -moz-border-radius: none;
    }
  }
}
